/*
====================
*Author: Jinit Patil
====================
*/
#include "mbed.h"

#define R 0x01                              //Command for reading data

SPI adxl355(SPI0_MOSI,SPI0_MISO,SPI0_SCLK); //Set SPI0 pins MOSI,MISO,SCLK
DigitalOut CS(SPI0_CS2);                    //SPI0_CS2 as output pin for chip Select
InterruptIn GPIO28(D8);                     //GPIO28 to capture interrupt generated by ADXL355
Serial uart(USBTX, USBRX,1000000);          //Open serial port at 1000000bps

/*Initialize variables*/
int32_t x_acc = 0;                          //Variable to store raw X-axis data
int32_t y_acc = 0;                          //Variable to store raw Y-axis data
int32_t z_acc = 0;                          //Variable to store raw Z-axis data
float x_acc_val = 0;                        //Variable to store X-axis data in 'g'
float y_acc_val = 0;                        //Variable to store Y-axis data in 'g'
float z_acc_val = 0;                        //Variable to store Z-axis data in 'g'
uint16_t temp = 0;                          //Variable to store temperature data
float temp_val = 0;                         //Variable to store temperature data in '°C'

bool data_rdy = 0;                          //Flag to indicate data is ready for reading

void GPIO28_ISR()                           //ISR for GPIO28
{
    data_rdy = 1;                           //Set data ready flag
}

int main()
{
    CS = 1;
    adxl355.frequency(1000000);             //SPI frequency at 1MHz
    adxl355.format(8,0);                    //8-bit,Mode 0
    CS = 0;                                 //Start SPI communication
    adxl355.write(0x28<<1);                 //Address of FILTER register
    adxl355.write(0x04);                    //Set ODR(Output Data Rate) = 250Hz and LPF cutoff = 62.5Hz
    CS = 1;                                 //Stop SPI communication
    CS = 0;
    adxl355.write(0x2D<<1);                 //Address of POWER CONTROL register
    adxl355.write(0x00);                    //Configure ADXL355 for measurement mode
    CS = 1;
    GPIO28.rise(&GPIO28_ISR);               //attach GPIO28 interrupt to GPIO28_ISR subroutine for rising edge
    
    while(1)
    {
        if(data_rdy)
        {
            data_rdy = 0;
            x_acc = y_acc = z_acc = 0;temp = 0;
            CS = 0;
            adxl355.write(0x06<<1|R);               //Address of TEMPERATURE DATA register
            temp = adxl355.write(0xFF)<<8;          //Read MSB of temperature data
            temp |= adxl355.write(0xFF);            //Read LSB of temperature data
            x_acc = adxl355.write(0xFF)<<24;        //Read X-axis data
            x_acc |= adxl355.write(0xFF)<<16;
            x_acc |= adxl355.write(0xFF)<<8;
            y_acc = adxl355.write(0xFF)<<24;        //Read Y-axis data
            y_acc |= adxl355.write(0xFF)<<16;
            y_acc |= adxl355.write(0xFF)<<8;
            z_acc = adxl355.write(0xFF)<<24;        //Read Z-axis data
            z_acc |= adxl355.write(0xFF)<<16;
            z_acc |= adxl355.write(0xFF)<<8;
            CS = 1;
            temp_val = (temp-1852)*1.0/-9.05+25;    //Temperature value in '°C'
            x_acc_val = x_acc/1073741824.0;         //X-axis acceleration data in 'g'
            y_acc_val = y_acc/1073741824.0;         //Y-axis acceleration data in 'g'
            z_acc_val = z_acc/1073741824.0;         //Z-axis acceleration data in 'g'
            //uart.printf("%f\t%f\t%f\n",x_acc_val,y_acc_val,z_acc_val);    //To plot values
            uart.printf("X=%f\tY=%f\tZ=%f\tT=%f\n",x_acc_val,y_acc_val,z_acc_val,temp_val); //To display values
        }
    }
}